/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.command.SubedCommandTest.java
 * Created on: Nov 18, 2016
 */
package net.bigeon.gclc.command;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import java.io.IOException;

import org.junit.Test;

import net.bigeon.gclc.command.Command;
import net.bigeon.gclc.command.ICommand;
import net.bigeon.gclc.command.SubedCommand;
import net.bigeon.gclc.command.base.MockCommand;
import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.InvalidCommandName;
import net.bigeon.gclc.manager.ConsoleInput;
import net.bigeon.gclc.manager.ConsoleOutput;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings("all")
public class SubedCommandTest {

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#add(net.bigeon.gclc.command.ICommand)}. */
    @Test
    public final void testAdd() {
        final SubedCommand cmd = new SubedCommand("name");

        try {
            cmd.add(new MockCommand("id"));
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
        }
        try {
            cmd.add(new MockCommand("id"));
            fail("addition of command with already used id succeeded");
        } catch (final InvalidCommandName e) {
            //
        }
        try {
            cmd.add(new MockCommand(""));
            fail("addition of command with invalid id succeeded");
        } catch (final InvalidCommandName e) {
            //
        }

    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#execute(ConsoleOutput, ConsoleInput, String...)}. */
    @Test
    public final void testExecute() {
        SubedCommand cmd = new SubedCommand("name");

        final MockCommand mock = new MockCommand("id");
        try {
            cmd.add(mock);
            cmd.add(new Command("fail") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    throw new CommandRunException("Failing command", null);
                }

                @Override
                public String tip() {
                    return null;
                }

                @Override
                protected String usageDetail() {
                    return null;
                }
            });
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
            assertNotNull(e);
        }

        try {
            cmd.execute(null, null, "id");
        } catch (final CommandRunException e) {
            fail("Unexpected exception when running mock command");
            assertNotNull(e);
        }
        try {
            cmd.execute(null, null, "fail");
            fail("Fail command error should be re thrown");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        try {
            cmd.execute(null, null);
            fail("Request for inexistent default command should fail");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        cmd = new SubedCommand("name", mock);

        try {
            cmd.add(mock);
            cmd.add(new Command("fail") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    throw new CommandRunException("Failing command");
                }

                @Override
                public String tip() {
                    return null;
                }

                @Override
                protected String usageDetail() {
                    return null;
                }
            });
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
            assertNotNull(e);
        }

        try {
            cmd.execute(null, null, "id");
        } catch (final CommandRunException e) {
            fail("Unexpected exception when running mock command");
            assertNotNull(e);
        }
        try {
            cmd.execute(null, null, "fail");
            fail("Fail command error should be re thrown");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        try {
            cmd.execute(null, null);
        } catch (final CommandRunException e) {
            fail("Request for default command should execute default command");
            assertNotNull(e);
        }
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#executeSub(ConsoleOutput, ConsoleInput, String, String...)}. */
    @Test
    public final void testExecuteSub() {
        final SubedCommand cmd = new SubedCommand("name");

        final MockCommand mock = new MockCommand("id");
        try {
            cmd.add(mock);
            cmd.add(new Command("fail") {

                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    throw new CommandRunException("Failing command");
                }

                @Override
                public String tip() {
                    return null;
                }

                @Override
                protected String usageDetail() {
                    return null;
                }
            });
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
            assertNotNull(e);
        }

        try {
            cmd.executeSub(null, null,"id");
        } catch (final CommandRunException e) {
            fail("Unexpected exception when running mock command");
            assertNotNull(e);
        }
        try {
            cmd.executeSub(null, null, "fail");
            fail("Fail command error should be re thrown");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#get(java.lang.String)}. */
    @Test
    public final void testGet() {
        final SubedCommand cmd = new SubedCommand("name");

        assertNull(cmd.get("id"));

        final MockCommand mock = new MockCommand("id");
        try {
            cmd.add(mock);
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
            assertNotNull(e);
        }
        assertEquals(mock, cmd.get("id"));
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#getCommandName()}. */
    @Test
    public final void testGetCommandName() {
        SubedCommand cmd = new SubedCommand("name");
        assertEquals("name", cmd.getCommandName());
        cmd = new SubedCommand("name with spaces");
        assertEquals("name with spaces", cmd.getCommandName());
        cmd = new SubedCommand("name", "some tip");
        assertEquals("name", cmd.getCommandName());
        cmd = new SubedCommand("name", new MockCommand(""));
        assertEquals("name", cmd.getCommandName());
        cmd = new SubedCommand("name", new MockCommand(""), "some tip");
        assertEquals("name", cmd.getCommandName());
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#help(ConsoleOutput, String...)}. */
    @Test
    public final void testHelp() {
        SubedCommand cmd = new SubedCommand("name");

        ICommand mock = new MockCommand("id");
        try {
            cmd.add(mock);
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
            assertNotNull(e);
        }

        try (PipedConsoleOutput manager = new PipedConsoleOutput()) {
            cmd.help(manager);
            assertEquals("\tid", manager.readNextLine());
            cmd.help(manager, "id");
            cmd.help(manager, "inexistent");
        } catch (final IOException e) {
            fail("Unexpected exception when running help");
            assertNotNull(e);
        }

        cmd = new SubedCommand("name", mock);

        try {
            cmd.add(mock);
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
            assertNotNull(e);
        }

        try (PipedConsoleOutput manager = new PipedConsoleOutput()) {
            cmd.help(manager);
            assertEquals("\tid", manager.readNextLine());
        } catch (final IOException e) {
            fail("Unexpected exception when running help");
            assertNotNull(e);
        }

        mock = new ICommand() {

            @Override
            public void execute(final ConsoleOutput out, final ConsoleInput in,
                                final String... args) throws CommandRunException {
                //
            }

            @Override
            public String getCommandName() {
                return "id";
            }

            @Override
            public void help(final ConsoleOutput manager,
                             final String... args) throws IOException {
                //
            }

            @Override
            public String tip() {
                return "tip";
            }
        };
        cmd = new SubedCommand("name", mock);

        try {
            cmd.add(mock);
        } catch (final InvalidCommandName e) {
            fail("addition of command with valid id failed");
            assertNotNull(e);
        }

        try (PipedConsoleOutput manager = new PipedConsoleOutput()) {
            cmd.help(manager);
            assertEquals("\ttip", manager.readNextLine());
            assertEquals("\tid: tip", manager.readNextLine());
        } catch (final IOException e) {
            fail("Unexpected exception when running help");
            assertNotNull(e);
        }
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#SubedCommand(java.lang.String)}. */
    @Test
    public final void testSubedCommandString() {
        SubedCommand cmd = new SubedCommand("name");
        assertEquals("name", cmd.getCommandName());
        cmd = new SubedCommand("name with spaces");
        assertNull(cmd.tip());
        assertEquals("name with spaces", cmd.getCommandName());
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#SubedCommand(java.lang.String, net.bigeon.gclc.command.ICommand)}. */
    @Test
    public final void testSubedCommandStringICommand() {
        SubedCommand cmd = new SubedCommand("name", new MockCommand(""));
        assertEquals("name", cmd.getCommandName());
        cmd = new SubedCommand("name with spaces", new MockCommand(""));
        assertNull(cmd.tip());
        assertEquals("name with spaces", cmd.getCommandName());
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#SubedCommand(java.lang.String, net.bigeon.gclc.command.ICommand, java.lang.String)}. */
    @Test
    public final void testSubedCommandStringICommandString() {
        SubedCommand cmd = new SubedCommand("name", new MockCommand(""), "tip");
        assertEquals("name", cmd.getCommandName());
        cmd = new SubedCommand("name with spaces", new MockCommand(""), "tip");
        assertEquals("name with spaces", cmd.getCommandName());
    }

    /** Test method for
     * {@link net.bigeon.gclc.command.SubedCommand#SubedCommand(java.lang.String, java.lang.String)}. */
    @Test
    public final void testSubedCommandStringString() {
        SubedCommand cmd = new SubedCommand("name", "tip");
        assertEquals("name", cmd.getCommandName());
        cmd = new SubedCommand("name with spaces", "tip");
        assertEquals("name with spaces", cmd.getCommandName());
    }

    /** Test method for {@link net.bigeon.gclc.command.SubedCommand#tip()}. */
    @Test
    public final void testTip() {
        SubedCommand cmd = new SubedCommand("name");
        assertNull(cmd.tip());
        cmd = new SubedCommand("name with spaces", new MockCommand(""), "tip");
        assertEquals("tip", cmd.tip());
    }
}
